// ---------------------------------------------------------------------------
// Programmierung von CAx-Systemen, Klasse für komplexe Zahlen
// 2018-05-15, T. Küpper
// ---------------------------------------------------------------------------

#define _USE_MATH_DEFINES  // für M_PI, M_E...

#include "compl.h"
#include <iostream>
#include <math.h>

// using namespace std; nur in der cpp-Datei aufrufen,
// aber nicht in der h-Datei!
using namespace std;

// Konstruktoren...
Complex::Complex()                         { Set(0, 0);       }
Complex::Complex(double real)              { Set(real, 0);    }
Complex::Complex(double real, double imag) { Set(real, imag); }

// Zugriff auf Attribute...
void Complex::Set(double real, double imag) { re = real; im = imag; }
double Complex::Real() { return re; }
double Complex::Imag() { return im; }

// Winkel (im Bogenmaß) der komplexen Zahl ermitteln...
double Complex::Arg()
{
    if(re > 0)
        return atan(im / re);
    else if(re == 0 && im > 0)
        return M_PI / 2.0;
    else if(re == 0 && im < 0)
        return -M_PI / 2.0;
    else if(re < 0 && im > 0)
        return atan(im / re) + M_PI;
    else
        return atan(im / re) - M_PI;
}

// Winkel (in Grad) der komplexen Zahl ermitteln...
double Complex::Argd()
{
    return Arg() * 180.0 / M_PI;
}

// Betrag der komplexen Zahl ermitteln...
double Complex::Abs()
{
    return sqrt(re * re + im * im);
}

// Zwei komplexe Zahlen addieren...
Complex operator+ (Complex a, Complex b)
{
    Complex sum;
    sum.Set(a.Real() + b.Real(), a.Imag() + b.Imag());
    return sum;
}

// Zwei komplexe Zahlen subtrahieren...
Complex operator- (Complex a, Complex b)
{
    Complex diff;
    diff.Set(a.Real() - b.Real(), a.Imag() - b.Imag());
    return diff;
}

// Zwei komplexe Zahlen multiplizieren...
Complex operator* (Complex a, Complex b)
{
    Complex result(
        a.Real() * b.Real() - a.Imag() * b.Imag(),
        a.Real() * b.Imag() + a.Imag() * b.Real());
    return result;
}

// Zwei komplexe Zahlen dividieren...
Complex operator/ (Complex a, Complex b)
{
    double denom = b.Real() * b.Real() + b.Imag() * b.Imag();
    double real1 = a.Real() * b.Real() + a.Imag() * b.Imag();
    double imag1 = a.Imag() * b.Real() - a.Real() * b.Imag();

    if(denom == 0)
        exit(-1);  // Division durch null --> Abbruch!

    Complex result(real1 / denom, imag1 / denom);
    return result;
}

// Exponentialfunktion, komplexer Exponent...
Complex exp(Complex exponent)
{
    double z = exponent.Abs();
    double phi = exponent.Arg();

    Complex result = pow(M_E, z * cos(phi));
    result = result * Complex(cos(z * sin(phi)), sin(z * sin(phi)));
    return result;
}

// Ausgabe einer komplexen Zahl...
ostream& operator<< (ostream& strm, Complex c)
{
    strm << "(" << c.Real();
    if(c.Imag() >= 0)
        strm << " + " << c.Imag() << "i)";
    else
        strm << " - " << -c.Imag() << "i)";
    return strm;
}

#define INPUT_START 0
#define INPUT_REAL  1
#define INPUT_SEPAR 2
#define INPUT_IMAG  3
#define INPUT_PAREN 4
#define INPUT_END   5

#define FORMAT_WITH_PARENS    1
#define FORMAT_WITHOUT_PARENS 2

// Eingabe einer komplexen Zahl...
std::istream& operator>> (std::istream& strm, Complex& c)
{
    int state = INPUT_START, format = FORMAT_WITH_PARENS;
    double re, im;
    char ch;

    while(state != INPUT_END)
    {
        switch(state)
        {
        case INPUT_START:
            state = INPUT_REAL, ch = 0, strm >> ch;
            if(ch != '(') strm.putback(ch), format = FORMAT_WITHOUT_PARENS;
            if(!strm.good()) state = INPUT_END;
            break;

        case INPUT_REAL:
            state = INPUT_END;
            if(strm >> re) state = INPUT_SEPAR;
            break;

        case INPUT_SEPAR:
            state = INPUT_IMAG, ch = 0, strm >> ch;
            if(ch != ',') strm.putback(ch);
            if(!strm.good()) state = INPUT_END;
            break;

        case INPUT_IMAG:
            state = INPUT_PAREN, strm >> im;
            if(!strm.good() || format == FORMAT_WITHOUT_PARENS) state = INPUT_END;
            break;

        case INPUT_PAREN:
            state = INPUT_END, ch = 0, strm >> ch;
            if(ch != ')') strm.putback(ch), strm.setstate(ios_base::failbit);
            break;
        }
    }

    if(strm.good()) c.Set(re, im);
    return strm;
}


