#define _USE_MATH_DEFINES

#include <iostream>
#include <vector>
#include <math.h>
#include <stdlib.h>
#include "chart.h"
#include "compl.h"

using namespace std;
typedef vector<Complex> Cvect;

// ---------------------------------------------------------------------------
// FFT für (komplexen) Vektor a berechnen.
// Achtung: Die Anzahl der Elemente im Vektor a muss eine Zweierpotenz sein!
// ---------------------------------------------------------------------------
Cvect fft(Cvect a)
{
    // Rekursionsabbruch?
    int n = a.size();
    if(n < 2) return a;

    // Teilvektoren a_i und a_ii erzeugen
    Cvect a_i, a_ii;
    for(auto x : a)
    {
        if(a_i.size() <= a_ii.size()) a_i .push_back(x);
        else                          a_ii.push_back(x);
    }

    // Rekursion, Selbstaufruf der Funktion fft()
    auto y_i  = fft(a_i );
    auto y_ii = fft(a_ii);

    // Elemente des Ergebnisvektors y berechnen
    Complex exponent(0, -2 * M_PI / n);
    Complex omega = 1, omega_n = exp(exponent);

    Cvect y(n);
    for(int k = 0; k < n / 2; k++)
    {
        y[k] = y_i[k] + omega * y_ii[k];
        y[k + n / 2] = y_i[k] - omega * y_ii[k];
        omega = omega * omega_n;
    }

    return y;
}

// ---------------------------------------------------------------------------
// Dieses Hauptprogramm wurde aus der MATLAB-Dokumentation ("doc fft")
// zur Funktion fft übernommen. Die entsprechenden MATLAB-Befehle sind
// als Kommentare im Quelltext angegeben.
// ---------------------------------------------------------------------------
int main(void)
{
    /*
        Fs = 1000;                    % Sampling frequency
        T = 1/Fs;                     % Sample time
        L = 1000;                     % Length of signal
        t = (0:L-1)*T;                % Time vector

        % Sum of a 50 Hz sinusoid and a 120 Hz sinusoid
        x = 0.7*sin(2*pi*50*t) + sin(2*pi*120*t);
        y = x + 2*randn(size(t));     % Sinusoids plus noise
    */

    Cvect y;
    double Fs = 1000, T = 1.0 / Fs, L = 1024, t = 0;
    while(y.size() < L)
    {
        double x = 0.7 * sin(2 * M_PI * 50 * t) + sin(2 * M_PI * 120 * t);
        double y_ = x + 2.0 * rand() / RAND_MAX - 1.0;
        chart_lineto(t, y_, CHART_RED);
        y.push_back(y_); t = t + T;
    }

    /*
        plot(Fs*t(1:50),y(1:50))
        title('Signal Corrupted with Zero-Mean Random Noise')
        xlabel('time (milliseconds)')
    */

    chart_axis_fmt("%g ms", "%g");
    chart_show(CHART_YZOOM_OFF, "Signal im Zeitbereich");

    /*
        NFFT = 2^nextpow2(L); % Next power of 2 from length of y
        Y = fft(y,NFFT)/L;
        f = Fs/2*linspace(0,1,NFFT/2+1);
    */

    Cvect Y = fft(y);

    /*
        % Plot single-sided amplitude spectrum.
        plot(f,2*abs(Y(1:NFFT/2+1)))
        title('Single-Sided Amplitude Spectrum of y(t)')
        xlabel('Frequency (Hz)')
        ylabel('|Y(f)|')
    */

    chart_clear();
    for(int idx = 0; idx <= L / 2; idx++)
        chart_lineto(Fs * idx / L, 2.0 * Y[idx].Abs() / L, CHART_BLUE);
    chart_axis_fmt("%g Hz", "%g");
    chart_show(CHART_YZOOM_OFF, "Amplitudenspektrum");
    return 0;
}

// ---------------------------------------------------------------------------
// Dieses "Hauptprogramm" führt einen ersten Test der FFT-Funktion durch.
// ---------------------------------------------------------------------------
int main1(void)
{
    Cvect y(8, 1);  // Vektor mit 8 Elementen, die alle den Wert 1 haben
    auto Y = fft(y);

    // Das erste Element ist gleich 8, die anderen Elemente sind gleich 0
    for(auto x : Y) cout << x << endl;

    /*
        % Die entsprechenden MATLAB-Befehle zum Vergleich:
        y = ones(1, 8);
        Y = fft(y);
        disp(Y);
    */

    return 0;
}


// ---------------------------------------------------------------------------
// Testprogramm zur Eingabe komplexer Zahlen von Eingabe-Streams.
// ---------------------------------------------------------------------------
int main2(void)
{
    Complex c;
    string s;
    cin >> c >> s;

    if(cin.good())
    {
        cout << "Eingabe: OK\n";
        cout << "c = " << c << endl;
        cout << "s = " << s << endl;
    }
    else
    {
        cout << "Eingabe: Fehler\n";
        cout << "c = " << c << endl;
        cout << "s = " << s << endl;
    }
    return 0;
}
